@@ -91,6 +91,7 @@
 #include <gtk/gtk.h>
 #include <string.h>
 #include <math.h>
+#include <locale.h>
 
 #include <libart_lgpl/art_affine.h>
 #include <libart_lgpl/art_misc.h>
@@ -346,7 +347,9 @@ gnome_print_pdf_new_with_paper (GnomePrinter *printer, const gchar *paper_name)
 		goto failure;
 
 	paper = gnome_paper_with_name (paper_name);
-	g_return_val_if_fail (paper != NULL, NULL);
+	if (!paper) {
+		g_warning ("file %s: line %d: Cannot find paper %s", __FILE__, __LINE__, paper_name);
+	}
 
 	pdf->paper = paper;
 	
@@ -423,11 +426,15 @@ gnome_print_pdf_write (GnomePrintContext *pc, const char *format, ...)
 	GnomePrintPdf *pdf;
 	va_list arguments;
 	gchar *text;
+	gchar *oldlocale;
 	
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
 	pdf = GNOME_PRINT_PDF (pc);
 	g_return_val_if_fail (pdf != NULL, -1);
 
+	oldlocale = g_strdup (setlocale (LC_NUMERIC, NULL));
+	setlocale (LC_NUMERIC, "C");
+		
 	va_start (arguments, format);
 	text = g_strdup_vprintf (format, arguments);
 	va_end (arguments);
@@ -435,6 +442,9 @@ gnome_print_pdf_write (GnomePrintContext *pc, const char *format, ...)
 	pdf->offset += gnome_print_context_write_file (pc, text, strlen(text));
 
 	g_free (text);
+
+	setlocale (LC_NUMERIC, oldlocale);
+	g_free (oldlocale);			
 	
 	return 0;
 }
@@ -738,6 +748,9 @@ gnome_print_pdf_page_write_contents (GnomePrintContext *pc, GnomePrintPdfPage *p
 																								 compr_type);
 		ret += gnome_print_pdf_write  (pc, EOL);
 	}	else {
+		ret += gnome_print_pdf_write  (pc,"/Length %i" EOL, page->stream_used); 
+		ret += gnome_print_pdf_write  (pc,">>" EOL);
+		ret += gnome_print_pdf_write  (pc,"stream" EOL);                
 		ret += gnome_print_pdf_write  (pc, "%s", page->stream);
 	}
 	
@@ -1616,9 +1629,8 @@ gnome_print_pdf_pages (GnomePrintContext *pc)
 		}
 	}
 
-	g_return_val_if_fail (pdf->paper != NULL, -1);
-	paper_width  = gnome_paper_pswidth (pdf->paper);
-	paper_height = gnome_paper_psheight (pdf->paper);
+	paper_width  = pdf->paper ? gnome_paper_pswidth (pdf->paper) : 21 * 72 / 2.54;
+	paper_height = pdf->paper ? gnome_paper_psheight (pdf->paper) : 29.7 * 72 / 2.54;
 	ret += gnome_print_pdf_write (pc,
 																"]" EOL
 																"/Count %i" EOL
@@ -1814,6 +1826,7 @@ gnome_print_pdf_write_content (GnomePrintPdf *pdf, const char *format, ...)
 	va_list arguments;
 	gchar *text;
 	gint text_length;
+	gchar *oldlocale;	
 
 	debug (FALSE, "");
 	
@@ -1822,10 +1835,16 @@ gnome_print_pdf_write_content (GnomePrintPdf *pdf, const char *format, ...)
 	pc = GNOME_PRINT_CONTEXT (pdf);
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
 
+	oldlocale = g_strdup (setlocale (LC_NUMERIC, NULL));
+	setlocale (LC_NUMERIC, "C");
+
 	va_start (arguments, format);
 	text = g_strdup_vprintf (format, arguments);
 	va_end (arguments);
 
+	setlocale (LC_NUMERIC, oldlocale);
+	g_free (oldlocale);
+
 	page = pdf->current_page;
 
 	text_length = strlen (text);
